#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include <string.h>
#include "ff.h"
#include "diskio.h"
     
volatile WORD Timer;	// 100Hz increment timer 
char buff[128];		// Read-write buffer
     
    ISR(TIMER2_COMP_vect)
    {	Timer++;			// Performance counter for this module 
    	disk_timerproc();	// Drive timer procedure of low level disk I/O module 
    }
     
    static void IOInit ()
    {	UBRRH = 0x00;		// Init UART baudrate 256000 bit per second
    	UBRRL = 0x01;
     
    	UCSRB = (1<<RXEN)|(1<<TXEN);	// TX, RX enable
    	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);
     
    	OCR2 = 90-1;		// Timer2: 100Hz interval (OC2)
    	TCCR2 = (1<<WGM21) | (1<<CS22) | (1<<CS20);
     
    	TIMSK = (1<<OCIE2);	// Enable TC2.oc, interrupt
    	sei();			// Enable interrupts
    }
     
    void uart_transmit(char data )	// Transmit one byte to UART
    { 
    while ( !( UCSRA & (1<<UDRE)) ); 
    UDR = data;
    }
     
    unsigned char uart_receive (void)	//Receive one byte from UART
    { while ( !(UCSRA & (1<<RXC)) ) ;
      return UDR; }
     
    void uart_transmit_message(char* msg)	//Transmit string to UART
    { unsigned char i;  i=0;
     
    while ((i<256)&(msg[i]!=0x00) )
    { uart_transmit(msg[i]);  i++; }
    }
     
    FRESULT scan_files (char* path)		//Scan files and directories on path
    {
     FRESULT res;
     FILINFO fno;
     DIR dir;
     int i;
     char *fn;
     
        res = f_opendir(&dir, path);
        if (res == FR_OK) {
                           i = strlen(path);
                           for (;;) 
    					    {
                             res = f_readdir(&dir, &fno);		//Read dir items
                             if (res != FR_OK || fno.fname[0] == 0) break;
                             if (fno.fname[0] == '.') continue;
                             fn = fno.fname;
                             if (fno.fattrib & AM_DIR) {		// Check item type, DIR or FILE
                                                        uart_transmit_message ("DIR: ");
                                                        uart_transmit_message (fn);
                                                        uart_transmit_message ("\r\n");
                                                       } 
    											  else {
                                                        uart_transmit_message ("FILE: ");
                                                        uart_transmit_message (fn);
                                                        uart_transmit_message ("\r\n");
                                                       }
                           }
                         }
        return res;
    }
     
    int main() 
    { FRESULT f_err_code;
      static FATFS FATFS_Obj;
      FIL fil_obj;
     
     IOInit (); // Init UART and Timer2
     
     disk_initialize(0); // Init drive	
     f_err_code=f_mount(0, &FATFS_Obj,1); //Mount Fat Fs
     uart_transmit_message ("mounting FAT ");
     if(f_err_code==0) 
    {uart_transmit_message ("OK\r\n");
     
    f_err_code=f_mkdir ("0:newdir");	// Create newdir
    uart_transmit_message ("creating newdir ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message("FAIL\r\n");
     
    f_err_code=f_chdir ("0:newdir");	// Set newdir to current directore
    uart_transmit_message ("change default dir to newdir ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message("FAIL\r\n");
     
    f_err_code=f_mkdir ("newdir2");		//Create newdir2 into newdir
    uart_transmit_message ("creating newdir2 into newdir ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message("FAIL\r\n");
     
    f_err_code=f_open(&fil_obj, "newfile.txt",FA_CREATE_NEW|FA_WRITE);	//Create newfile into newdir
    uart_transmit_message ("creating newfile.txt into newdir ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message("FAIL\r\n");
     
    f_puts ("creating and writing ok if you see this\r",&fil_obj);		//Writing to newfile
    uart_transmit_message ("writing data to newfile.txt \r\n");
     
    f_err_code=f_close(&fil_obj);		// Close newfile
    uart_transmit_message ("closing newfile.txt ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message ("FAIL\r\n");
     
    f_err_code=f_open(&fil_obj, "newfile.txt",FA_READ);	//Open newfile for reading
    uart_transmit_message ("open newfile.txt ");
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message("FAIL\r\n");
     
    uart_transmit_message ("data in newfile.txt:\r\n ");	
    uart_transmit_message (f_gets (buff,128,&fil_obj)); //Read data from newfile
     
    f_err_code=f_close(&fil_obj);
    uart_transmit_message ("closing newfile.txt ");	//Close newfile
    if(f_err_code==0) uart_transmit_message ("OK\r\n");
    else  uart_transmit_message ("FAIL\r\n");
     
    uart_transmit_message("scaning newdir:\r\n");
    scan_files ("0:/newdir"); //Scan newdir
    uart_transmit_message("scaning ended\r\n");
     
    uart_transmit_message("remove newdir2 into newdir? y/n \r\n");	
    if (uart_receive()=='y') 	
    {f_err_code=f_unlink ("newdir2");	// Delete newdir2
     uart_transmit_message ("remoing newdir2 ");
     if(f_err_code==0) uart_transmit_message ("OK\r\n");
     else uart_transmit_message("FAIL\r\n");  }
     
    uart_transmit_message("remove newfile.txt into newdir? y/n \r\n");
    if (uart_receive()=='y') 	
    {f_err_code=f_unlink ("newfile.txt");	//Delete newfile
     uart_transmit_message ("remoing newfile ");
     if(f_err_code==0) uart_transmit_message ("OK\r\n");
     else uart_transmit_message("FAIL\r\n"); }
     
    //f_err_code=f_mount(0, NULL);			//Unmount Fat Fs
    uart_transmit_message ("unmounting FAT ");
    if(f_err_code==0) 
    uart_transmit_message ("OK\r\n");
    else  
    uart_transmit_message("FAIL\r\n"); }
    else 
    uart_transmit_message("FAIL\r\n");
    while (1); return 0;
}
